@@ -1,7 +1,7 @@
 use v6;
 use fatal;
 
-unit class Getopt::Long:ver<0.3.1>;
+unit class Getopt::Long:ver<0.3.4>;
 
 class Exception is CORE::Exception {
 	has Str:D $.message is required;
@@ -10,7 +10,7 @@ class Exception is CORE::Exception {
 	}
 }
 
-class ValueInvalid is CORE::Exception {
+role FormattableException is CORE::Exception {
 	has Str:D $.format is required;
 	method new(Str $format) {
 		return self.bless(:$format);
@@ -18,16 +18,15 @@ class ValueInvalid is CORE::Exception {
 	method rethrow-with(Str $name) {
 		die Exception.new($!format.sprintf($name));
 	}
+	method message() {
+		$!format.sprintf('some');
+	}
 }
 
-class ConverterInvalid is CORE::Exception {
-	has $.format;
-	method new(Str $format) {
-		return self.bless(:$format);
-	}
-	method rethrow-with(Str $name) {
-		die Exception.new($!format.sprintf($name));
-	}
+class ValueInvalid does FormattableException {
+}
+
+class ConverterInvalid does FormattableException {
 }
 
 my sub convert(Code:D $converter, Str:D $value) {
@@ -42,6 +41,9 @@ my sub convert(Code:D $converter, Str:D $value) {
 		when X::Temporal {
 			die ValueInvalid.new(.message.subst(/'string ' ( \' .* \' ) <.before ';'> /, { "$0 given as %s argument" }));
 		}
+		when FormattableException {
+			.rethrow;
+		}
 		default {
 			die ValueInvalid.new("Can not convert %s argument \"$value\": {.message}");
 		}
@@ -109,10 +111,8 @@ my class Option {
 	}
 }
 
-has Code:D @!positionals;
-has Option:D %!options;
-
-submethod BUILD(:@!positionals, :%!options) { }
+has Code:D @!positionals is built;
+has Option:D %!options is built;
 
 method !positionals {
 	return @!positionals.map(*.returns);
@@ -147,6 +147,7 @@ my %converter-for-type{Any:U} = (
 	Pair.new(Rat,      *.Rat),
 	Pair.new(Num,      *.Num),
 	Pair.new(Real,     *.Real),
+	Pair.new(Numeric,  *.Numeric),
 	Pair.new(Complex,  *.Complex),
 	Pair.new(Str,      *.Str),
 	Pair.new(IO::Path, *.IO),
@@ -248,8 +249,7 @@ method new-from-patterns(Getopt::Long:U: @patterns, Str:D :$positionals = "") {
 			for $match.ast.kv -> $key, $option {
 				%options{$key} = $option;
 			}
-		}
-		else {
+		} else {
 			die Exception.new("Couldn't parse argument specification '$pattern'");
 		}
 		CATCH { when ConverterInvalid {
@@ -264,20 +264,17 @@ my sub get-converter(Any:U $type) {
 	state $coercion-how = try ::("Metamodel::CoercionHOW");
 	if %converter-for-type{$type} -> &converter {
 		return &converter;
-	}
-	elsif $type.HOW ~~ $coercion-how {
+	} elsif $type.HOW ~~ $coercion-how {
 		my &primary = get-converter($type.^constraint_type());
 		return sub coercion-converter(Str $input) {
 			return $type.^coerce(primary($input));
 		}
-	}
-	elsif $type.HOW ~~ Metamodel::EnumHOW {
+	} elsif $type.HOW ~~ Metamodel::EnumHOW {
 		my $valid-values = $type.WHO.keys.sort({ $type.WHO{$^value} }).join(", ");
 		return sub enum-converter(Str $value) {
 			return $type.WHO{$value} // die ValueInvalid.new(qq{Can't convert %s argument "$value" to $type.^name(), valid values are: $valid-values});
 		}
-	}
-	else {
+	} else {
 		die ConverterInvalid.new("No argument conversion known for %s argument (type {$type.^name})");
 	}
 }
@@ -291,8 +288,7 @@ multi sub trait_mod:<is>(Parameter $param, Str:D :getopt(:$option)!) is export(:
 	with Argument.parse($option, :rule('argument')) -> $match {
 		my %options = make-option($param.named_names, |$match.ast);
 		return $param does Formatted(:%options);
-	}
-	else {
+	} else {
 		die Exception.new("Couldn't parse parameter {$param.name}'s argument specification '$option'");
 	}
 }
@@ -325,21 +321,18 @@ my multi get-named(&candidate) {
 	for &candidate.signature.params.grep(*.named) -> $param {
 		if $param ~~ Formatted {
 			@options.append: $param.options;
-		}
-		else {
+		} else {
 			my @names = $param.named_names;
 			if $param.sigil eq '$' {
 				my $type = $param.type;
 				my $constraints = $param.constraints;
 				if $param.type === Bool {
 					@options.append: make-option(@names, ScalarStore, { :$constraints }, 0..0, {}, ?$param.default)
-				}
-				else {
+				} else {
 					my $converter = get-converter($param.type);
 					@options.append: make-option(@names, ScalarStore, { :$converter, :$constraints }, 1..1);
 				}
-			}
-			else {
+			} else {
 				my $type = $param.type.of ~~ Any ?? $param.type.of !! Any;
 				my $converter = get-converter($type);
 				@options.append: make-option(@names, %store-for{$param.sigil}, { :$type, :$converter }, 1..1);
@@ -380,7 +373,7 @@ method new-from-sub(Getopt::Long:U: Sub $main) {
 	return self.new(:%options, :@positionals);
 }
 
-method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = False, :$compat-builtin = False, :named-anywhere(:$permute) = !$compat-builtin, :$bundling = !$compat-builtin, :$compat-singles = $compat-builtin, :$compat-negation = $compat-builtin, :$compat-positional = $compat-builtin, :$auto-help = $compat-builtin, :$write-args) {
+method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = False, :$compat-builtin = False, :named-anywhere(:$permute) = !$compat-builtin, :$bundling = !$compat-builtin, :$compat-singles = $compat-builtin, :$compat-negation = $compat-builtin, :$compat-positional = $compat-builtin, :$compat-space = $compat-builtin, :$auto-help = $compat-builtin, :$write-args) {
 	my @list;
 
 	while @args {
@@ -391,23 +384,18 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = Fa
 		sub get-option(Str:D $key, Str:D $name) {
 			with %!options{$key} -> $option {
 				return $option;
-			}
-			elsif $key eq 'help' && $auto-help {
+			} elsif $key eq 'help' && $auto-help {
 				return Option.new(:store(ScalarStore.new(:key<help>)), :arity(0..0), :default);
-			}
-			elsif $auto-abbreviate {
+			} elsif $auto-abbreviate {
 				my @names = %!options.keys.grep(*.starts-with($key));
 				if @names == 1 {
 					return %!options{ @names[0] };
-				}
-				elsif @names > 1 {
+				} elsif @names > 1 {
 					die Exception.new("Ambiguous partial option $name, possible interpretations: @names[]");
-				}
-				else {
+				} else {
 					die Exception.new("Unknown option $name");
 				}
-			}
-			else {
+			} else {
 				die Exception.new("Unknown option $name");
 			}
 		}
@@ -422,38 +410,38 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = Fa
 				take-value($option, @args.shift, $name);
 			}
 
-			while @args && $consumed < $option.arity.max && !@args[0].starts-with('--') {
+			while !$compat-space && @args && $consumed < $option.arity.max && !@args[0].starts-with('--') {
 				take-value($option, @args.shift, $name);
 			}
 
 			if $consumed == 0 && $option.arity.min == 0 {
 				$option.store-default(%hash);
-			}
-			elsif $consumed < $option.arity.min {
+			} elsif $consumed < $option.arity.min {
 				die Exception.new("The argument $name requires a value but none was specified");
 			}
 		}
 
 		my rule name { [\w+]+ % '-' | '?' }
 
-		if $compat-singles && $head ~~ / ^ '-' <name> '=' $<value>=[.*] / -> $/ {
-			my $option = get-option(~$<name>, "-$<name>");
-			die Exception.new("-$<name> doesn't take an argument") if $option.arity.max != 1;
-			take-value($option, ~$<value>, "-$<name>");
-		}
-		elsif $bundling && $head ~~ / ^ '-' $<values>=[\w .* ] $ / -> $/ {
+		if $bundling && $head ~~ / ^ '-' $<values>=[\w .* ] $ / -> $/ {
 			my @values = $<values>.Str.comb;
 			for @values.keys -> $index {
 				my $value = @values[$index];
 				my $option = get-option($value, "-$value");
 				if $option.arity.max > 0 && $index + 1 < @values.elems {
-					take-value($option, $<values>.substr($index + 1), "-$value");
+					my $offset = $compat-singles && @values[$index + 1] eq '=' ?? 2 !! 1;
+					take-value($option, $<values>.substr($index + $offset), "-$value");
 				}
 
-				take-args($option, $value);
+				take-args($option, "-$value");
 				last if $consumed;
 			}
 		}
+		elsif $compat-singles && $head ~~ / ^ '-' <name> '=' $<value>=[.*] / -> $/ {
+			my $option = get-option(~$<name>, "-$<name>");
+			die Exception.new("-$<name> doesn't take an argument") if $option.arity.max != 1;
+			take-value($option, ~$<value>, "-$<name>");
+		}
 		elsif $head eq '--' {
 			@list.append: |@args;
 			last;
@@ -472,16 +460,13 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = Fa
 				my $option = get-option(~$<name>, ~$<full-name>);
 				die Exception.new("Option $<full-name> doesn't take an argument") if $option.arity.max != 1;
 				take-value($option, ~$<value> but False, ~$<full-name>);
-			}
-			else {
+			} else {
 				take-args(get-option('no-' ~ $<name>, ~$<full-name>), ~$<full-name>);
 			}
-		}
-		else {
+		} else {
 			if $permute {
 				@list.push: $head;
-			}
-			else {
+			} else {
 				@list.append: $head, |@args;
 				last;
 			}
@@ -548,18 +533,15 @@ our sub ARGS-TO-CAPTURE(Sub $func, @args) is export(:DEFAULT, :MAIN) {
 	CATCH { when Exception { note .message; &*EXIT(2) } };
 }
 
-our &MAIN_HELPER is export(:DEFAULT, :MAIN) = $*PERL.compiler.version after 2018.06
-	?? anon sub MAIN_HELPER(Bool $in-is-args, $retval = 0) {
-		if $in-is-args {
-			my $in := $*IN;
-			my $*ARGFILES := IO::ArgFiles.new($in, :nl-in($in.nl-in), :chomp($in.chomp), :encoding($in.encoding), :bin(!$in.encoding));
-			call-main(callframe(1), $retval);
-		}
-		else {
-			call-main(callframe(1), $retval);
-		}
+our sub MAIN_HELPER(Bool $in-is-args, $retval = 0) is export(:DEFAULT, :MAIN) {
+	if $in-is-args {
+		my $in := $*IN;
+		my $*ARGFILES := IO::ArgFiles.new($in, :nl-in($in.nl-in), :chomp($in.chomp), :encoding($in.encoding), :bin(!$in.encoding));
+		call-main(callframe(1), $retval);
+	} else {
+		call-main(callframe(1), $retval);
 	}
-	!! anon sub MAIN_HELPER($retval = 0) { call-main(callframe(1), $retval); }
+}
 
 =begin pod
 
@@ -661,6 +643,7 @@ It supports the following types for named and positional arguments:
 =item Rat
 =item Num
 =item Real
+=item Numeric
 =item Complex
 =item IO::Path
 =item DateTime
@@ -988,14 +971,21 @@ Path (C<IO::Path>). For example C<foo/bar.txt>.
 =begin item2
 d
 
-A date and time (C<DateTime>). For example C<2019-12-30T01:23:45-0700>.
+An ISO-8601 formatted date and time (C<DateTime>). For example C<2019-12-30T01:23:45-0700>.
 
 =end item2
 
 =begin item2
 a
 
-A Date (C<Date>). For example C<2019-12-30>.
+A ISO-8601 formatted date (C<Date>). For example C<2019-12-30>.
+
+=end item2
+
+=begin item2
+v
+
+A Version (C<Version>). For example C<1.2.3>.
 
 =end item2
 
@@ -1156,6 +1146,17 @@ possible.
 
 =end item
 
+=begin item
+compat-space (default: C<$compat-builtin>)
+
+By default, an option with an optional argument will take that as a
+separate argument unless that argument starts with C<-->; e.g.
+C<--foo bar>. If this option is enabled, no such separate arguments are
+allowed, and the only way to express such an argument is in the same
+argument: C<--foo=bar>.
+
+=end item
+
 =begin item
 auto-help (default: C<$compat-builtin>)
 
